@@ -26,6 +26,7 @@ my $dbpassword = 'rootpass';
 
 my $dbh = DBI->connect("DBI:mysql:radius",$dbusername, $dbpassword);
 my $sth;
+my $DEBUG = 0;
 
 require 'timelocal.pl';
 
@@ -81,8 +82,8 @@ while (<>) {
 	   $DETAIL{'NAS-Port-Type'} = $1;
 	}
 
-	$DETAIL{'Time'} = timelocal ($6, $5, $4, $3, $months{$2}, $7) if
-	(/^(\w{3})\s(\w{3})\s{1,2}(\d{1,2})\s(\d{2}):(\d{2}):(\d{2})\s19(\d{2})\n/
+	$DETAIL{'Time'} = timelocal ($6, $5, $4, $3, $months{$2}, $7 - 1900) if
+	(/^(\w{3})\s(\w{3})\s{1,2}(\d{1,2})\s(\d{2}):(\d{2}):(\d{2})\s(\d{4})\n/
 	&& (grep {$1 eq $_} @wdays) && (grep {$2 eq $_} keys %months));
 	
 	if (/Acct-Session-Time = (\d+)/) {
@@ -135,23 +136,29 @@ while (<>) {
 
 	if ($DETAIL{'Acct-Status-Type'} eq 'Start') {
 		$sth = $dbh->prepare("SELECT RadAcctId FROM radacct WHERE AcctSessionId = '$DETAIL{'Acct-Session-Id'}' AND UserName = '$DETAIL{'User-Name'}' AND NASIPAddress = '$DETAIL{'NAS-IP-Address'}' AND NASPortId = '$DETAIL{'NAS-Port-Id'}' AND AcctStartTime = 0");
+		print "SELECT RadAcctId FROM radacct WHERE AcctSessionId = '$DETAIL{'Acct-Session-Id'}' AND UserName = '$DETAIL{'User-Name'}' AND NASIPAddress = '$DETAIL{'NAS-IP-Address'}' AND NASPortId = '$DETAIL{'NAS-Port-Id'}' AND AcctStartTime = 0\n" if $DEBUG;
 		$sth->execute || print "Error: ".$dbh->errstr();
 		($startsessid) = $sth->fetchrow_array;
 		$sth->finish;
 		if ($startsessid) {
 			$sth = $dbh->do("UPDATE radacct SET AcctStartTime = FROM_UNIXTIME('$DETAIL{'Time'}'), AcctStartDelay = '$DETAIL{'Acct-Delay-Time'}' WHERE RadAcctId = '$startsessid'") || print "Error: ".$dbh->errstr();
+			print "UPDATE radacct SET AcctStartTime = FROM_UNIXTIME('$DETAIL{'Time'}'), AcctStartDelay = '$DETAIL{'Acct-Delay-Time'}' WHERE RadAcctId = '$startsessid'\n" if $DEBUG;
 		} else {
 			$sth = $dbh->do("INSERT INTO radacct VALUES ('','$DETAIL{'Acct-Session-Id'}','$DETAIL{'User-Name'}','','$DETAIL{'NAS-IP-Address'}',$DETAIL{'NAS-Port-Id'},'$DETAIL{'NAS-Port-Type'}',FROM_UNIXTIME('$DETAIL{'Time'}'), 0, 0, '$DETAIL{'Acct-Authentic'}','$DETAIL{'Connect-Info'}',0, 0, '$DETAIL{'Called-Station-Id'}','$DETAIL{'Calling-Station-Id'}','','$DETAIL{'Service-Type'}','$DETAIL{'Framed-Protocol'}','$DETAIL{'Framed-IP-Address'}',$DETAIL{'Acct-Delay-Time'}, 0)") || print "Error: ".$dbh->errstr();
+			print "INSERT INTO radacct VALUES ('','$DETAIL{'Acct-Session-Id'}','$DETAIL{'User-Name'}','','$DETAIL{'NAS-IP-Address'}',$DETAIL{'NAS-Port-Id'},'$DETAIL{'NAS-Port-Type'}',FROM_UNIXTIME('$DETAIL{'Time'}'), 0, 0, '$DETAIL{'Acct-Authentic'}','$DETAIL{'Connect-Info'}',0, 0, '$DETAIL{'Called-Station-Id'}','$DETAIL{'Calling-Station-Id'}','','$DETAIL{'Service-Type'}','$DETAIL{'Framed-Protocol'}','$DETAIL{'Framed-IP-Address'}',$DETAIL{'Acct-Delay-Time'}, 0)\n" if $DEBUG;
 		}
 	} elsif ($DETAIL{'Acct-Status-Type'} eq 'Stop') {
 		$sth = $dbh->prepare("SELECT RadAcctId FROM radacct WHERE AcctSessionId = '$DETAIL{'Acct-Session-Id'}' AND UserName = '$DETAIL{'User-Name'}' AND NASIPAddress = '$DETAIL{'NAS-IP-Address'}' AND NASPortId = '$DETAIL{'NAS-Port-Id'}' AND AcctStopTime = 0") || print "Error: ".$dbh->errstr();
+		print "SELECT RadAcctId FROM radacct WHERE AcctSessionId = '$DETAIL{'Acct-Session-Id'}' AND UserName = '$DETAIL{'User-Name'}' AND NASIPAddress = '$DETAIL{'NAS-IP-Address'}' AND NASPortId = '$DETAIL{'NAS-Port-Id'}' AND AcctStopTime = 0\n" if $DEBUG;
 		$sth->execute || print "Error: ".$dbh->errstr();
 		($stopsessid) = $sth->fetchrow_array;
 		$sth->finish;
 		if ($stopsessid) {
 			$sth = $dbh->do("UPDATE radacct SET AcctStopTime = FROM_UNIXTIME('$DETAIL{'Time'}'), AcctStopDelay = '$DETAIL{'Acct-Delay-Time'}', AcctSessionTime = $DETAIL{'Acct-Session-Time'}, AcctInputOctets = $DETAIL{'Acct-Input-Octets'}, AcctOutputOctets = $DETAIL{'Acct-Output-Octets'}, AcctTerminateCause = '$DETAIL{'Acct-Terminate-Cause'}' WHERE RadAcctId = '$stopsessid'") || print "Error: ".$dbh->errstr();
+			print "UPDATE radacct SET AcctStopTime = FROM_UNIXTIME('$DETAIL{'Time'}'), AcctStopDelay = '$DETAIL{'Acct-Delay-Time'}', AcctSessionTime = $DETAIL{'Acct-Session-Time'}, AcctInputOctets = $DETAIL{'Acct-Input-Octets'}, AcctOutputOctets = $DETAIL{'Acct-Output-Octets'}, AcctTerminateCause = '$DETAIL{'Acct-Terminate-Cause'}' WHERE RadAcctId = '$stopsessid'\n" if $DEBUG;
 		} else {
 			$sth = $dbh->do("INSERT INTO radacct VALUES ('','$DETAIL{'Acct-Session-Id'}','$DETAIL{'User-Name'}','','$DETAIL{'NAS-IP-Address'}',$DETAIL{'NAS-Port-Id'},'$DETAIL{'NAS-Port-Type'}', 0, FROM_UNIXTIME('$DETAIL{'Time'}'), $DETAIL{'Acct-Session-Time'}, '$DETAIL{'Acct-Authentic'}','$DETAIL{'Connect-Info'}',$DETAIL{'Acct-Input-Octets'}, $DETAIL{'Acct-Output-Octets'}, '$DETAIL{'Called-Station-Id'}','$DETAIL{'Calling-Station-Id'}','$DETAIL{'Acct-Terminate-Cause'}','$DETAIL{'Service-Type'}','$DETAIL{'Framed-Protocol'}','$DETAIL{'Framed-IP-Address'}', 0, $DETAIL{'Acct-Delay-Time'})") || print "Error: ".$dbh->errstr();
+			print "INSERT INTO radacct VALUES ('','$DETAIL{'Acct-Session-Id'}','$DETAIL{'User-Name'}','','$DETAIL{'NAS-IP-Address'}',$DETAIL{'NAS-Port-Id'},'$DETAIL{'NAS-Port-Type'}', 0, FROM_UNIXTIME('$DETAIL{'Time'}'), $DETAIL{'Acct-Session-Time'}, '$DETAIL{'Acct-Authentic'}','$DETAIL{'Connect-Info'}',$DETAIL{'Acct-Input-Octets'}, $DETAIL{'Acct-Output-Octets'}, '$DETAIL{'Called-Station-Id'}','$DETAIL{'Calling-Station-Id'}','$DETAIL{'Acct-Terminate-Cause'}','$DETAIL{'Service-Type'}','$DETAIL{'Framed-Protocol'}','$DETAIL{'Framed-IP-Address'}', 0, $DETAIL{'Acct-Delay-Time'})\n" if $DEBUG;
 		}
 	}
 }
