@@ -207,8 +207,7 @@ int radzap(UINT4 nasaddr, int port, char *user, time_t t)
 /*
  *	Store logins in the RADIUS utmp file.
  */
-int rad_accounting_new(AUTH_REQ *authreq, int dowtmp)
-{
+int rad_accounting_new(AUTH_REQ *authreq, int dowtmp) {
 	struct radutmp	ut, u;
 	struct utmp	wt;
 	VALUE_PAIR	*vp;
@@ -536,18 +535,7 @@ int rad_accounting_new(AUTH_REQ *authreq, int dowtmp)
 
 
 
-int rad_accounting_orig(AUTH_REQ *authreq, int authtype, char *f)
-{
-#ifndef USEMYSQL
-	FILE		*outfd;
-	char		nasname[128];
-	char		buffer[512];
-	char		*s;
-	UINT4		nas;
-	NAS		*cl;
-	long		curtime;
-	struct stat	st;
-#else
+int rad_accounting_orig(AUTH_REQ *authreq, int authtype, char *f, SQLSOCK *socket) {
 	time_t		nowtime;
 	struct tm	*tim;
         char		datebuf[20];
@@ -556,77 +544,17 @@ int rad_accounting_orig(AUTH_REQ *authreq, int authtype, char *f)
 	FILE		*backupfile;
 	struct stat	backup;
 	char		*valbuf;
-	MYSQLREC sqlrecord = {"", "", "", "", 0, "", "", 0, "", 0, "", "", 0, 0, "", "", "", "", "", "", 0};
-	MYSQLREC backuprecord = {"", "", "",  "", 0, "", "", 0, "", 0, "", "", 0, 0, "", "", "", "", "", "", 0};
-#endif
+	char		username[AUTH_STRING_LEN*2+1];
+	SQLREC sqlrecord = {"", "", "", "", 0, "", "", 0, "", 0, "", "", 0, 0, "", "", "", "", "", "", 0};
+	SQLREC backuprecord = {"", "", "",  "", 0, "", "", 0, "", 0, "", "", 0, 0, "", "", "", "", "", "", 0};
 	VALUE_PAIR	*pair;
 	int		ret = 0;
+	int		length;
 
-
-#ifndef USEMYSQL
-        /*
-         *      See if we have an accounting directory. If not,
-         *      return.
-         */
-        if (stat(radacct_dir, &st) < 0)
-                return 0;
-        curtime = time(0);
-
-        /*
-         *      Find out the name of this terminal server. We try
-         *      to find the PW_NAS_IP_ADDRESS in the naslist file.
-         *      If that fails, we look for the originating address.
-         *      Only if that fails we resort to a name lookup.
-         */
-        cl = NULL;
-        nas = authreq->ipaddr;
-        if ((pair = pairfind(authreq->request, PW_NAS_IP_ADDRESS)) != NULL)
-                nas = pair->lvalue;
-        if (authreq->server_ipaddr)
-                nas = authreq->server_ipaddr;
-
-        if ((cl = nas_find(nas)) != NULL) {
-                if (cl->shortname[0])
-                        strcpy(nasname, cl->shortname);
-                else
-                        strcpy(nasname, cl->longname);
-        }
-
-        if (cl == NULL) {
-                s = ip_hostname(nas);
-                if (strlen(s) >= sizeof(nasname) || strchr(s, '/'))
-                        return -1;
-                strcpy(nasname, s);
-        }
-
-        /*
-         *      Create a directory for this nas.
-         */
-        sprintf(buffer, "%.250s/%.250s", radacct_dir, nasname);
-        (void) mkdir(buffer, 0755);
-
-        /*
-         *      Write Detail file.
-         */
-        sprintf(buffer, "%.200s/%.200s/%.100s", radacct_dir, nasname, f ? f : "detail");
-        if((outfd = fopen(buffer, "a")) == (FILE *)NULL) {
-                log(L_ERR, "Acct: Couldn't open file %s", buffer);
-                ret = -1;
-        } else {
-
-                /* Post a timestamp */
-                fputs(ctime(&curtime), outfd);
-#endif
- 
 	pair = authreq->request;
-#ifdef USEMYSQL
 	strncpy(sqlrecord.Realm, authreq->realm, SQLBIGREC);
-#endif
 	while(pair != (VALUE_PAIR *)NULL) {
 
-				
-#ifdef USEMYSQL
-
            /* Check the pairs to see if they are anything we are interested in. */
             switch(pair->attribute) {
                 case PW_ACCT_SESSION_ID:
@@ -634,7 +562,12 @@ int rad_accounting_orig(AUTH_REQ *authreq, int authtype, char *f)
                 	break;
                 	
                 case PW_USER_NAME:
-                	strncpy(sqlrecord.UserName, pair->strvalue, SQLBIGREC);
+			if (strlen(pair->strvalue) > AUTH_STRING_LEN)
+				length = AUTH_STRING_LEN;
+			else
+				length = strlen(pair->strvalue);
+			sql_escape_string(username, pair->strvalue, length);
+                	strncpy(sqlrecord.UserName, username, SQLBIGREC);
                 	break;
                 	
                 case PW_NAS_IP_ADDRESS:
@@ -723,42 +656,10 @@ int rad_accounting_orig(AUTH_REQ *authreq, int authtype, char *f)
                 default:
                 	break;
 		}
-#else 
-		if (pair->attribute != PW_PASSWORD) {
-			fputs("\t", outfd);
-			fprint_attr_val(outfd, pair);
-			fputs("\n", outfd);
-		}
-#endif
-
 
 		pair = pair->next;
 	}
 
-#ifndef USEMYSQL
-
-                /*
-                 *      Add non-protocol attibutes.
-                 */
-                fprintf(outfd, "\tTimestamp = %ld\n", curtime);
-                switch(authtype) {
-                    case 0:
-                        fputs("\tRequest-Authenticator = Verified\n", outfd);
-                        break;
-                    case 1:
-                        fputs("\tRequest-Authenticator = None\n", outfd);
-                        break;
-                    case 2:
-                        fputs("\tRequest-Authenticator = Unverified\n", outfd);
-                        break;
-                    default:
-                        break;
-                }
-                fputs("\n", outfd);
-                fclose(outfd);
-	}
-
-#else
 
         nowtime = time(0) - sqlrecord.AcctDelayTime;
         tim = localtime(&nowtime);
@@ -795,11 +696,11 @@ int rad_accounting_orig(AUTH_REQ *authreq, int authtype, char *f)
 				log(L_INFO, "Acct:  Clearing out sql backup file - %s", MYSQLBACKUP);
 
 				while(!feof(backupfile)) {
-					if(fread(&backuprecord, sizeof(MYSQLREC), 1, backupfile) == 1) {
+					if(fread(&backuprecord, sizeof(SQLREC), 1, backupfile) == 1) {
 
 						/* pass our filled structure to the
 					 	   function that will write to the database */
-						ret = mysql_save_acct(&backuprecord);
+						ret = sql_save_acct(socket, &backuprecord);
 
 					}
 
@@ -809,12 +710,7 @@ int rad_accounting_orig(AUTH_REQ *authreq, int authtype, char *f)
 			}
 		}
 	}
-	ret = mysql_save_acct(&sqlrecord);
-	if (!mysql_keepopen) {
-		mysql_close(MyAcctSock);
-		MyAcctSock = NULL;
-	}
-#endif
+	ret = sql_save_acct(socket, &sqlrecord);
 
 	return ret;
 }
@@ -823,7 +719,7 @@ int rad_accounting_orig(AUTH_REQ *authreq, int authtype, char *f)
 /*
  *	rad_accounting: call both the old and new style accounting functions.
  */
-int rad_accounting(AUTH_REQ *authreq, int activefd)
+int rad_accounting(AUTH_REQ *authreq, int activefd, SQLSOCK *socket)
 {
 	int reply = 0;
 	int auth;
@@ -848,14 +744,14 @@ int rad_accounting(AUTH_REQ *authreq, int activefd)
 		 */
 		if (rad_accounting_new(authreq, doradwtmp) == 0)
 			reply = 1;
-		if (rad_accounting_orig(authreq, auth, NULL) == 0)
+		if (rad_accounting_orig(authreq, auth, NULL, socket) == 0)
 			reply = 1;
 	} else {
 		/*
 		 *	First log into the details file, before the
 		 *	username gets stripped by rad_accounting_new.
 		 */
-		if (rad_accounting_orig(authreq, auth, NULL) == 0)
+		if (rad_accounting_orig(authreq, auth, NULL, socket) == 0)
 			reply = 1;
 		if (rad_accounting_new(authreq, doradwtmp) == 0)
 			reply = 1;
@@ -874,194 +770,3 @@ int rad_accounting(AUTH_REQ *authreq, int activefd)
 	return reply ? 0 : -1;
 }
 
-
-#ifndef USEMYSQL
-/*
- *	Timeout handler (10 secs)
- */
-static int got_alrm;
-static void alrm_handler()
-{
-	got_alrm = 1;
-}
-
-/*
- *	Check one terminal server to see if a user is logged in.
- */
-static int rad_check_ts(struct radutmp *ut)
-{
-	int	pid, st, e;
-	int	n;
-	NAS	*nas;
-	char	address[16];
-	char	port[8];
-	char	session_id[12];
-	char	*s;
-	void	(*handler)(int);
-
-	/*
-	 *	Find NAS type.
-	 */
-	if ((nas = nas_find(ntohl(ut->nas_address))) == NULL) {
-		log(L_ERR, "Accounting: unknown NAS");
-		return -1;
-	}
-
-	/*
-	 *	Fork.
-	 */
-	handler = signal(SIGCHLD, SIG_DFL);
-	if ((pid = fork()) < 0) {
-		log(L_ERR, "Accounting: fork: %s", strerror(errno));
-		signal(SIGCHLD, handler);
-		return -1;
-	}
-
-	if (pid > 0) {
-		/*
-		 *	Parent - Wait for checkrad to terminate.
-		 *	We timeout in 10 seconds.
-		 */
-		got_alrm = 0;
-		signal(SIGALRM, alrm_handler);
-		alarm(10);
-		while((e = waitpid(pid, &st, 0)) != pid)
-			if (e < 0 && (errno != EINTR || got_alrm))
-				break;
-		alarm(0);
-		signal(SIGCHLD, handler);
-		if (got_alrm) {
-			kill(pid, SIGTERM);
-			sleep(1);
-			kill(pid, SIGKILL);
-			log(L_ERR, "Check-TS: timeout waiting for checkrad");
-			return 2;
-		}
-		if (e < 0) {
-			log(L_ERR, "Check-TS: unknown error in waitpid()");
-			return 2;
-		}
-		return WEXITSTATUS(st);
-	}
-
-	/*
-	 *	Child - exec checklogin with the right parameters.
-	 */
-	for (n = 32; n >= 3; n--)
-		close(n);
-
-	ipaddr2str(address, ntohl(ut->nas_address));
-	sprintf(port, "%d", ut->nas_port);
-	sprintf(session_id, "%.8s", ut->session_id);
-
-	s = CHECKRAD2;
-	execl(CHECKRAD2, "checkrad", nas->nastype, address, port,
-		ut->login, session_id, NULL);
-	if (errno == ENOENT) {
-		s = CHECKRAD1;
-		execl(CHECKRAD1, "checklogin", nas->nastype, address, port,
-			ut->login, session_id, NULL);
-	}
-	log(L_ERR, "Check-TS: exec %s: %s", s, strerror(errno));
-
-	/*
-	 *	Exit - 2 means "some error occured".
-	 */
-	exit(2);
-}
-#endif
-
-/*
- *	See if a user is already logged in.
- *
- *	Check twice. If on the first pass the user exceeds his
- *	max. number of logins, do a second pass and validate all
- *	logins by querying the terminal server (using eg. telnet).
- *
- *	Returns: 0 == OK, 1 == double logins, 2 == multilink attempt
- */
-#ifndef USEMYSQL
-int rad_check_multi(char *name, VALUE_PAIR *request, int maxsimul)
-{
-	FILE		*wfp;
-	int		fd;
-	int		count;
-	struct radutmp	u, empty;
-	struct utmp	wt;
-	VALUE_PAIR	*fra;
-	int		mpp = 1;
-	UINT4		ipno = 0;
-
-	if ((fd = open(RADUTMP, O_CREAT|O_RDWR, 0644)) < 0)
-		return 0;
-
-	/*
-	 *	We don't lock in the first pass.
-	 */
-	count = 0;
-	while(read(fd, &u, sizeof(u)) == sizeof(u))
-		if (strncmp(name, u.login, RUT_NAMESIZE) == 0
-		    && u.type == P_LOGIN)
-			count++;
-
-	if (count < maxsimul) {
-		close(fd);
-		return 0;
-	}
-	lseek(fd, (off_t)0, SEEK_SET);
-
-	/*
-	 *	Setup some stuff, like for MPP detection.
-	 */
-	if ((fra = pairfind(request, PW_FRAMED_IP_ADDRESS)) != NULL)
-		ipno = htonl(fra->lvalue);
-	memset(&empty, 0, sizeof(empty));
-
-	/*
-	 *	lockf() the file while reading/writing.
-	 */
-#if defined(F_LOCK) && !defined(BSD)
-		(void)lockf(fd, F_LOCK, LOCK_LEN);
-#else
-		(void)flock(fd, LOCK_EX);
-#endif
-
-	/*
-	 *	Allright, there are too many concurrent logins.
-	 *	Check all registered logins by querying the
-	 *	terminal server directly.
-	 */
-	count = 0;
-	while (read(fd, &u, sizeof(u)) == sizeof(u)) {
-		if (strncmp(name, u.login, RUT_NAMESIZE) == 0
-		    && u.type == P_LOGIN) {
-			if (rad_check_ts(&u) == 1) {
-				count++;
-				/*
-				 *	Does it look like a MPP attempt?
-				 */
-				if (strchr("SCPA", u.proto) &&
-				    ipno && u.framed_address == ipno)
-					mpp = 2;
-			}
-			else {
-				/*
-				 *	False record - zap it.
-				 */
-
-				lseek(fd, -(off_t)sizeof(u), SEEK_CUR);
-				write(fd, &empty, sizeof(empty));
-
-				if ((wfp = fopen(RADWTMP, "a")) != NULL) {
-					make_wtmp(&u, &wt, PW_STATUS_STOP);
-					fwrite(&wt, sizeof(wt), 1, wfp);
-					fclose(wfp);
-				}
-			}
-		}
-	}
-	close(fd);
-
-	return (count < maxsimul) ? 0 : mpp;
-}
-#endif
