@@ -648,7 +648,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 				sql_select_query(socket, querystr);
 				row = sql_fetch_row(socket);
 				num = atol(row[0]);
-				sql_free_result(socket);
+				sql_finish_select_query(socket);
 				if (num >= check_item->lvalue) {
 					sprintf(umsg, "\r\nYour realm has reached it's limit of %d simultaneous logins\r\n\n", (int)check_item->lvalue);
 					user_msg = umsg;
@@ -674,7 +674,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 					used = 0;
 				else
 					used = atol(row[0]);
-				sql_free_result(socket);
+				sql_finish_select_query(socket);
 				
 				if (used >= allowed) {
 					sprintf(umsg, "\r\nYour realm has reached it's total limit of %d seconds\r\n\n", (int)check_item->lvalue);
@@ -720,7 +720,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 					used = 0;
 				else
 					used = atol(row[0]);
-				sql_free_result(socket);
+				sql_finish_select_query(socket);
 				
 				if (used >= allowed) {
 					sprintf(umsg, "\r\nYour realm has reached it's limit of %d seconds\r\n\n", (int)check_item->lvalue);
@@ -766,7 +766,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 					used = 0;
 				else
 					used = atol(row[0]);
-				sql_free_result(socket);
+				sql_finish_select_query(socket);
 				
 				if (used >= allowed) {
 					sprintf(umsg, "\r\nYour realm has reached it's limit of %d hours\r\n\n", (int)check_item->lvalue);
@@ -905,6 +905,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 	}
 
 
+	/* FIXME: Shouldn't this be a paircmp() somewhere to check all nas items? */
 	if (result >=0 && (check_item = pairfind(user_check, PW_NAS_PORT_TYPE)) != NULL) {
 
 		VALUE_PAIR *req;
@@ -927,6 +928,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 			
 		}
 	}
+
 	/*       Check if the user is calling from the specified calling station id */
 	if (result >=0 && (check_item = pairfind(user_check, PW_CALLING_STATION_ID)) != NULL) {
 
@@ -962,7 +964,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 			used = 0;
 		else
 			used = atol(row[0]);
-		sql_free_result(socket);
+		sql_finish_select_query(socket);
 		
 		if (used >= allowed) {
 			sprintf(umsg, "\r\nYour account has reached it total limit of %d seconds\r\n\n", (int)check_item->lvalue);
@@ -1008,7 +1010,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 			used = 0;
 		else
 			used = atol(row[0]);
-		sql_free_result(socket);
+		sql_finish_select_query(socket);
 
 		if (used >= allowed) {
 			sprintf(umsg, "\r\nYour account has reached it limit of %d seconds\r\n\n", (int)check_item->lvalue);
@@ -1054,7 +1056,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 			used = 0;
 		else
 			used = atol(row[0]);
-		sql_free_result(socket);
+		sql_finish_select_query(socket);
 		
 		if (used >= allowed) {
 			sprintf(umsg, "\r\nYour account has reached it limit of %d hours\r\n\n", (int)check_item->lvalue);
@@ -1277,8 +1279,8 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 		if (strcmp(namepair->strvalue, "miquels") == 0)
 			strcpy(userpass, "guess");
 #endif
-		log(L_AUTH, "Login OK: [%s%s%s] (%s)", namepair->strvalue, log_auth_pass ? "/" : "",
-			log_auth_pass ? userpass : "", auth_name(authreq, 0));
+		log(L_AUTH, "Login OK: [%s%s%s] (%s) socket %d", namepair->strvalue, log_auth_pass ? "/" : "",
+			log_auth_pass ? userpass : "", auth_name(authreq, 0), socket->id);
 	}
 	if (exec_program && !exec_wait) {
 		/*
