@@ -29,17 +29,20 @@
  *
  *	Function: mysql_start
  *
- *	Purpose: Reads in MySQL Config File 
+ *	Purpose: Reads SQL Config File 
  *
  *************************************************************************/
 
-int sql_init(void) {
+int sql_init(int reload) {
 	FILE    *sqlfd;
         char    dummystr[64];
         char    namestr[64];
         int     line_no = 0;
         char    buffer[256];
         char    sqlfile[256];
+
+	if (reload)
+		free(sql->config);
 	if ((sql->config = malloc(sizeof(SQL_CONFIG))) == NULL) {
 		log(L_ERR|L_CONS, "no memory");
 		exit(1);
@@ -57,6 +60,7 @@ int sql_init(void) {
 	strcpy(sql->config->mysql_acct_table,"radacct");
 	sql->config->sqltrace = 0;
 	sql->config->sensitiveusername = 1;
+	sql->config->deletestalesessions = 0;
 
 	strcpy(sql->config->mysql_nas_table,"nas");
 	strcpy(sql->config->mysql_realm_table, "realms");
@@ -81,7 +85,7 @@ int sql_init(void) {
                 if(strncasecmp(buffer, "server", 6) == 0) {
                         /* Read the SERVER line */
                         if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
-                               log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
                        } else {
                          strcpy(sql->config->mysql_server,namestr);
                        }
@@ -89,7 +93,7 @@ int sql_init(void) {
                 if(strncasecmp(buffer, "login", 5) == 0) {
                         /* Read the LOGIN line */
                         if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
-                               log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
                        } else {
                          strcpy(sql->config->mysql_login,namestr);
                        }
@@ -97,7 +101,7 @@ int sql_init(void) {
                 if(strncasecmp(buffer, "password", 8) == 0) {
                         /* Read the PASSWORD line */
                         if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
-                               log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
                        } else {
                          strcpy(sql->config->mysql_password,namestr);
                        }
@@ -105,7 +109,7 @@ int sql_init(void) {
                 if(strncasecmp(buffer, "radius_db", 9) == 0) {
                         /* Read the RADIUS_DB line */
                         if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
-                               log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
                        } else {
                          strcpy(sql->config->mysql_db,namestr);
                        }
@@ -113,7 +117,7 @@ int sql_init(void) {
                 if(strncasecmp(buffer, "authcheck_table", 15) == 0) {
                         /* Read the AUTHCHECK_TABLE line */
                         if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
-                               log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
                        } else {
                          strncpy(sql->config->mysql_authcheck_table,namestr, MAX_TABLE_LEN);
                        }
@@ -121,7 +125,7 @@ int sql_init(void) {
                 if(strncasecmp(buffer, "authreply_table", 15) == 0) {
                         /* Read the AUTHREPLY_TABLE line */
                         if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
-                               log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
                        } else {
                          strncpy(sql->config->mysql_authreply_table,namestr, MAX_TABLE_LEN);
                        }
@@ -129,7 +133,7 @@ int sql_init(void) {
                 if(strncasecmp(buffer, "groupcheck_table", 16) == 0) {
                         /* Read the GROUPCHECK_TABLE line */
                         if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
-                               log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
                        } else {
                          strncpy(sql->config->mysql_groupcheck_table,namestr, MAX_TABLE_LEN);
                        }
@@ -137,7 +141,7 @@ int sql_init(void) {
                 if(strncasecmp(buffer, "groupreply_table", 16) == 0) {
                         /* Read the GROUP_REPLY_TABLE line */
                         if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
-                               log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
                        } else {
                          strncpy(sql->config->mysql_groupreply_table,namestr, MAX_TABLE_LEN);
                        }
@@ -145,7 +149,7 @@ int sql_init(void) {
                 if(strncasecmp(buffer, "usergroup_table", 15) == 0) {
                         /* Read the USERGROUP_TABLE line */
                         if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
-                               log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
                        } else {
                          strncpy(sql->config->mysql_usergroup_table,namestr, MAX_TABLE_LEN);
                        }
@@ -153,7 +157,7 @@ int sql_init(void) {
                 if(strncasecmp(buffer, "realmgroup_table", 16) == 0) {
                         /* Read the REALMGROUP_TABLE line */
                         if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
-                               log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
                        } else {
                          strncpy(sql->config->mysql_realmgroup_table,namestr, MAX_TABLE_LEN);
                        }
@@ -161,7 +165,7 @@ int sql_init(void) {
                 if(strncasecmp(buffer, "acct_table", 10) == 0) {
                         /* Read the ACCT_TABLE line */
                         if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
-                               log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
                        } else {
                          strncpy(sql->config->mysql_acct_table,namestr, MAX_TABLE_LEN);
                        }
@@ -169,7 +173,7 @@ int sql_init(void) {
                 if(strncasecmp(buffer, "nas_table", 9) == 0) {
                         /* Read the NAS_TABLE line */
                         if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
-                               log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
                        } else {
                          strncpy(sql->config->mysql_nas_table,namestr, MAX_TABLE_LEN);
                        }
@@ -177,7 +181,7 @@ int sql_init(void) {
                 if(strncasecmp(buffer, "realm_table", 9) == 0) {
                        /* Read the REALM_TABLE line */
                        if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
-                               log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
                       } else {
                          strncpy(sql->config->mysql_realm_table,namestr, MAX_TABLE_LEN);
                       }
@@ -185,7 +189,7 @@ int sql_init(void) {
                 if(strncasecmp(buffer, "dict_table", 9) == 0) {
                         /* Read the DICT_TABLE line */
                         if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
-                               log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
                        } else {
                          strncpy(sql->config->mysql_dict_table,namestr, MAX_TABLE_LEN);
                        }
@@ -193,7 +197,7 @@ int sql_init(void) {
                 if(strncasecmp(buffer, "sqltrace", 8) == 0) {
                         /* Read the SQLTRACE line */
                         if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
-                               log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
                       } else {
                          if(strncasecmp(namestr, "on", 2) == 0) {
                            sql->config->sqltrace = 1;
@@ -205,7 +209,7 @@ int sql_init(void) {
                if(strncasecmp(buffer, "sensitiveusername", 17) == 0) {
                         /* Read the SENSITIVEUSERNAME line */
                         if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
-                               log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
                        } else {
                          if(strncasecmp(namestr, "on", 2) == 0) {
                            sql->config->sensitiveusername = 1;
@@ -214,13 +218,25 @@ int sql_init(void) {
                         }
                        }
                }
+               if(strncasecmp(buffer, "deletestalesessions", 19) == 0) {
+                        /* Read the DELETESTALESESSIONS line */
+                        if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
+                       } else {
+                         if(strncasecmp(namestr, "on", 2) == 0) {
+                           sql->config->deletestalesessions = 1;
+                        } else {
+                           sql->config->deletestalesessions = 0;
+                        }
+                       }
+               }
 
        }
        fclose(sqlfd);
        
        log(L_INFO,"SQL: Attempting to connect to %s@%s:%s", sql->config->mysql_login, sql->config->mysql_server, sql->config->mysql_db);
 
-       sql_init_socket();
+       sql_init_socket(reload);
            
        return 0;
 }
@@ -228,7 +244,7 @@ int sql_init(void) {
 
 /*************************************************************************
  *
- *	Function: mysql_get_socket
+ *	Function: sql_get_socket
  *
  *	Purpose: Return a SQL socket from the connection pool           
  *
@@ -252,7 +268,7 @@ SQLSOCK *sql_get_socket(void) {
 
 /*************************************************************************
  *
- *	Function: mysql_release_socket
+ *	Function: sql_release_socket
  *
  *	Purpose: Frees a SQL socket back to the connection pool           
  *
@@ -267,13 +283,13 @@ int sql_release_socket(SQLSOCK *socket) {
  
 /*************************************************************************
  *
- *	Function: mysql_save_acct
+ *	Function: sql_save_acct
  *
  *	Purpose: Write data from the sqlrecord structure to the database
  *
  *************************************************************************/
 
-int sql_save_acct(SQLSOCK *socket, MYSQLREC *sqlrecord) {
+int sql_save_acct(SQLSOCK *socket, SQLREC *sqlrecord) {
 
 	char		querystr[2048];
 	FILE		*mysqlfile;
@@ -319,6 +335,7 @@ int sql_save_acct(SQLSOCK *socket, MYSQLREC *sqlrecord) {
 
        	 if (sql_query(socket, querystr) < 0)
 	      log(L_ERR, "Acct: Couldn't update SQL accounting after NAS reboot - %s", sql_error(socket));
+	 sql_finish_query(socket);
 
          if (mysqlfile) {
               fputs(querystr, mysqlfile);
@@ -331,7 +348,8 @@ int sql_save_acct(SQLSOCK *socket, MYSQLREC *sqlrecord) {
 	if (sqlrecord->AcctStatusTypeId == PW_STATUS_ALIVE) {
 		sprintf(querystr, "UPDATE %s SET FramedIPAddress = '%s' WHERE AcctSessionId = '%s' AND UserName = '%s' AND NASIPAddress= '%s'", sql->config->mysql_acct_table, sqlrecord->FramedIPAddress, sqlrecord->AcctSessionId, sqlrecord->UserName, sqlrecord->NASIPAddress);
 		if (sql_query(socket, querystr) < 0)
-		log(L_ERR, "Acct: Couldn't update SQL accounting for ALIVE packet - %s", sql_error(socket));
+			log(L_ERR, "Acct: Couldn't update SQL accounting for ALIVE packet - %s", sql_error(socket));
+		sql_finish_query(socket);
 
 		if (mysqlfile) {
 			fputs(querystr, mysqlfile);
@@ -355,6 +373,7 @@ int sql_save_acct(SQLSOCK *socket, MYSQLREC *sqlrecord) {
              );
        	     if (sql_query(socket, querystr) < 0)
 	        log(L_ERR, "Acct: Couldn't update SQL accounting START record - %s", sql_error(socket));
+	     sql_finish_query(socket);
 
              num = sql_affected_rows(socket);
              if (num == 0) {
@@ -381,15 +400,23 @@ int sql_save_acct(SQLSOCK *socket, MYSQLREC *sqlrecord) {
 
        	        if (sql_query(socket, querystr) < 0)
 	   	  log(L_ERR, "Acct: Couldn't insert SQL accounting START record - %s", sql_error(socket));
+		sql_finish_query(socket);
              }
 
            /* Got stop record */
            } else {
 
+		/* If stop but zero session length drop it as in invalid packet */
+		/* This is to fix CISCO's aaa from filling our table with bogus crap */
+		if (sqlrecord->AcctSessionTime <= 0) {
+			log(L_ERR, "Acct: Invalid STOP record. [%s] STOP record but zero session length? (nas %s)", sqlrecord->UserName, sqlrecord->NASIPAddress);
+			return 0;
+		}
+
              sprintf(querystr, "SELECT RadAcctId FROM %s WHERE AcctSessionId='%s' AND NASIPAddress='%s' AND UserName='%s'", sql->config->mysql_acct_table, sqlrecord->AcctSessionId, sqlrecord->NASIPAddress, sqlrecord->UserName);
               sql_select_query(socket, querystr);
               num = sql_num_rows(socket);
-	      sql_free_result(socket);
+	      sql_finish_select_query(socket);
 
              if (num > 0) {
 
@@ -410,6 +437,7 @@ int sql_save_acct(SQLSOCK *socket, MYSQLREC *sqlrecord) {
 
        	        if (sql_query(socket, querystr) < 0)
 	           log(L_ERR, "Acct: Couldn't update SQL accounting STOP record - %s", sql_error(socket));
+		sql_finish_query(socket);
 
              } else if (num == 0) {
 
@@ -440,6 +468,7 @@ int sql_save_acct(SQLSOCK *socket, MYSQLREC *sqlrecord) {
 
        	        if (sql_query(socket, querystr) < 0)
 		   log(L_ERR, "Acct: Couldn't insert SQL accounting STOP record - %s", sql_error(socket));
+		sql_finish_query(socket);
              }
 
           }
@@ -636,11 +665,11 @@ int sql_getvpdata(SQLSOCK *socket, char *table, VALUE_PAIR **vp, char *user, int
 
 		if (sql_userparse(vp, row) != 0) {
 	 		log(L_ERR|L_CONS, "Error getting data from SQL");
-			sql_free_result(socket);
+			sql_finish_select_query(socket);
 			return -1;
 		}
 	}
-	sql_free_result(socket);
+	sql_finish_select_query(socket);
 
 	return rows;
 
@@ -768,7 +797,7 @@ int sql_check_multi(SQLSOCK *socket, char *name, VALUE_PAIR *request, int maxsim
 	sql_select_query(socket, querystr);
 	row = sql_fetch_row(socket);
 	count = atoi(row[0]);
-	sql_free_result(socket);
+	sql_finish_select_query(socket);
 
 	if (count < maxsimul)
 		return 0;
@@ -780,7 +809,7 @@ int sql_check_multi(SQLSOCK *socket, char *name, VALUE_PAIR *request, int maxsim
 		ipno = htonl(fra->lvalue);
 
 	count = 0;
-	sprintf(querystr, "SELECT * FROM %s WHERE UserName = '%s' AND AcctStopTime = 0", sql->config->mysql_acct_table, name);
+	sprintf(querystr, "SELECT * FROM %s WHERE %s AND AcctStopTime = 0", sql->config->mysql_acct_table, authstr);
 	sql_select_query(socket, querystr);
 	while ((row = sql_fetch_row(socket))) {
 		int check = sql_check_ts(row);
@@ -797,14 +826,19 @@ int sql_check_multi(SQLSOCK *socket, char *name, VALUE_PAIR *request, int maxsim
 			 *	False record - zap it
 			 */
 
-			/* FIXME: We need our own connection because we are in a fetch loop for the accounting stuff */
-   			/*log(L_ERR,"Deleteing stale session [%s] (from nas %s/%s)", row[2], row[4], row[5]);
-			sprintf(querystr, "DELETE FROM %s WHERE RadAcctId = '%s'", sql->config->mysql_acct_table, row[0]);
-			sql_query(socket, querystr);
-			*/
+			if (sql->config->deletestalesessions) {
+				SQLSOCK *socket;
+
+   				log(L_ERR,"Deleteing stale session [%s] (from nas %s/%s)", row[2], row[4], row[5]);
+				socket = sql_get_socket();
+				sprintf(querystr, "DELETE FROM %s WHERE RadAcctId = '%s'", sql->config->mysql_acct_table, row[0]);
+				sql_query(socket, querystr);
+				sql_finish_query(socket);
+				sql_close_socket(socket);
+			}
 		}
 	}
-	sql_free_result(socket);
+	sql_finish_select_query(socket);
 
 	return (count < maxsimul) ? 0 : mpp; 
 
@@ -851,7 +885,7 @@ int sql_read_naslist(SQLSOCK *socket) {
        	        c->next = naslist;
                	naslist = c;
        	}
-	sql_free_result(socket);
+	sql_finish_select_query(socket);
 
         return 0;
 } 
@@ -890,7 +924,7 @@ int sql_read_clients(SQLSOCK *socket) {
        	        c->next = clients;
                	clients = c;
        	}
-	sql_free_result(socket);
+	sql_finish_select_query(socket);
 
         return 0;
 } 
@@ -961,7 +995,7 @@ int sql_dict_init(SQLSOCK *socket) {
 				log(L_ERR|L_CONS,
 		"dict_init: Invalid name length on line %d of dictionary",
 					line_no);
-				sql_free_result(socket);
+				sql_finish_select_query(socket);
 				return(-1);
 			}
 
@@ -969,7 +1003,7 @@ int sql_dict_init(SQLSOCK *socket) {
 				log(L_ERR|L_CONS,
 			"dict_init: Invalid value on line %d of dictionary",
 					line_no);
-				sql_free_result(socket);
+				sql_finish_select_query(socket);
 				return(-1);
 			}
 			if (row[3][0] != '0')
@@ -993,7 +1027,7 @@ int sql_dict_init(SQLSOCK *socket) {
 				log(L_ERR|L_CONS,
 			"dict_init: Invalid type on line %d of dictionary",
 					line_no);
-				sql_free_result(socket);
+				sql_finish_select_query(socket);
 				return(-1);
 			}
 
@@ -1005,7 +1039,7 @@ int sql_dict_init(SQLSOCK *socket) {
 				log(L_ERR|L_CONS,
 			"dict_init: unknown vendor %s on line %d of dictionary",
 				row[5], line_no);
-				sql_free_result(socket);
+				sql_finish_select_query(socket);
 				return -1;
 			}
 
@@ -1013,7 +1047,7 @@ int sql_dict_init(SQLSOCK *socket) {
 			if((attr = (DICT_ATTR *)malloc(sizeof(DICT_ATTR))) ==
 					(DICT_ATTR *)NULL) {
 				log(L_ERR|L_CONS, "dict_init: out of memory");
-				sql_free_result(socket);
+				sql_finish_select_query(socket);
 				return(-1);
 			}
 			strcpy(attr->name, row[2]);
@@ -1041,7 +1075,7 @@ int sql_dict_init(SQLSOCK *socket) {
 				log(L_ERR|L_CONS,
 		"dict_init: Invalid attribute length on line %d of dictionary",
 					line_no);
-				sql_free_result(socket);
+				sql_finish_select_query(socket);
 				return(-1);
 			}
 
@@ -1049,7 +1083,7 @@ int sql_dict_init(SQLSOCK *socket) {
 				log(L_ERR|L_CONS,
 		"dict_init: Invalid name length on line %d of dictionary",
 					line_no);
-				sql_free_result(socket);
+				sql_finish_select_query(socket);
 				return(-1);
 			}
 
@@ -1057,7 +1091,7 @@ int sql_dict_init(SQLSOCK *socket) {
 				log(L_ERR|L_CONS,
 			"dict_init: Invalid value on line %d of dictionary",
 					line_no);
-				sql_free_result(socket);
+				sql_finish_select_query(socket);
 				return(-1);
 			}
 			value = atoi(row[4]);
@@ -1066,7 +1100,7 @@ int sql_dict_init(SQLSOCK *socket) {
 			if((dval = (DICT_VALUE *)malloc(sizeof(DICT_VALUE))) ==
 					(DICT_VALUE *)NULL) {
 				log(L_ERR|L_CONS, "dict_init: out of memory");
-				sql_free_result(socket);
+				sql_finish_select_query(socket);
 				return(-1);
 			}
 			strcpy(dval->attrname, row[2]);
@@ -1087,7 +1121,7 @@ int sql_dict_init(SQLSOCK *socket) {
 				log(L_ERR|L_CONS,
 		"dict_init: Invalid attribute length on line %d of dictionary",
 					line_no);
-				sql_free_result(socket);
+				sql_finish_select_query(socket);
 				return(-1);
 			}
 
@@ -1095,14 +1129,14 @@ int sql_dict_init(SQLSOCK *socket) {
 				log(L_ERR|L_CONS,
 			"dict_init: Invalid value on line %d of dictionary",
 					line_no);
-				sql_free_result(socket);
+				sql_finish_select_query(socket);
 				return(-1);
 			}
 			value = atoi(row[3]);
 
 			/* Create a new VENDOR entry for the list */
 			if (addvendor(row[2], value) < 0) {
-				sql_free_result(socket);
+				sql_finish_select_query(socket);
 				return -1;
 			}
 #ifdef ATTRIB_NMC
@@ -1111,7 +1145,7 @@ int sql_dict_init(SQLSOCK *socket) {
 #endif
 		}
 	}
-	sql_free_result(socket);
+	sql_finish_select_query(socket);
 	return(0);
 }
 
@@ -1146,6 +1180,8 @@ int sql_read_realms(SQLSOCK *socket) {
 	opts1[0] = opts2[0] = 0;
 	c->realm[0] = 0;
 	c->server[0] = 0;
+	c->striprealm = 1;
+	c->dohints = 0;
 
 	if (c->realm != NULL)
 		strNcpy(c->realm, row[0], sizeof(c->realm));
@@ -1173,7 +1209,7 @@ int sql_read_realms(SQLSOCK *socket) {
 	realms = c;
     }
        
-    sql_free_result(socket);
-    return 0;
+    sql_finish_select_query(socket);
+    return 1;
 }
 
