@@ -120,8 +120,11 @@ static void reread_config(int reload) {
 	}
 
 
-	if (sql_init() != 0) {
-		log(L_ERR,"SQL Error: SQL connections could not be initialized");
+	if (sql_init(reload) != 0) {
+		if (!reload) 
+			log(L_ERR,"SQL Error: SQL connections could not be initialized");
+		else
+			log(L_ERR,"SQL Error: SQL connections could not be re-initialized");
 		res = -1;
 	}
 
