@@ -19,14 +19,21 @@
  *	Purpose: Connect to the sql server
  *
  *************************************************************************/
-int sql_init_socket(void) {
+int sql_init_socket(int reload) {
 
 	int i;
 
+	/* Clear up old connections if reload */
+	if (reload)
+	        for (i = 0; i < sql->config->max_sql_socks; i++)
+			if (!sql_close_socket(sql->socks[i]))
+				log(L_CONS|L_ERR, "Could not release socket %d", i);
+	
+
         /* Initalize our connection pool */
         for (i = 0; i < sql->config->max_sql_socks; i++) {
 	 	if ((sql->socks[i] = sql_create_socket()) == NULL) {
-	   		DEBUG2("SQL: Failed to connect socket %d", i);
+	   		log(L_CONS|L_ERR, "SQL: Failed to connect socket %d", i);
 		} else {
 	   		sql->socks[i]->id = i;
            		sql->socks[i]->in_use = 0;
@@ -61,6 +68,20 @@ SQLSOCK *sql_create_socket(void) {
 	}
 	return socket;
 }
+
+/*************************************************************************
+ *
+ *	Function: sql_close_socket
+ *
+ *	Purpose: Close and free a sql socket
+ *
+ *************************************************************************/
+int sql_close_socket(SQLSOCK *socket) {
+	DEBUG2("SQL: Closing socket %d", socket->id);
+	mysql_close(socket->sock);
+	free(socket);
+	return 1;
+}
  
 /*************************************************************************
  *
@@ -91,9 +112,9 @@ int sql_select_query(SQLSOCK *socket, char *querystr) {
 	DEBUG(querystr);
  mysql_query(socket->sock, querystr);
  if (sql_store_result(socket) && sql_num_fields(socket)) 
-	return 0;
- else
 	return 1;
+ else
+	return 0;
 
 }
 
@@ -110,7 +131,7 @@ int sql_store_result(SQLSOCK *socket) {
 
 	if (!(socket->result = mysql_store_result(socket->sock))) {
 		log(L_ERR,"MYSQL Error: Cannot get result");
-		log(L_ERR,"MYSQL error: %s",mysql_error(socket->sock));
+		log(L_ERR,"MYSQL Error: %s",mysql_error(socket->sock));
 		sql_close(socket);
 		return 0;
 	}
@@ -130,7 +151,11 @@ int sql_store_result(SQLSOCK *socket) {
 int sql_num_fields(SQLSOCK *socket) {
 
 	int	num = 0;
+#if MYSQL_VERSION_ID >= 32224
+	if (!(num = mysql_field_count(socket->sock))) {
+#else
 	if (!(num = mysql_num_fields(socket->sock))) {
+#endif
 		log(L_ERR,"MYSQL Error: Cannot get result");
 		log(L_ERR,"MYSQL error: %s",mysql_error(socket->sock));
 		sql_close(socket);
@@ -261,17 +286,11 @@ int sql_affected_rows(SQLSOCK *socket) {
  *
  *      Function: sql_escape_string
  *
- *      Purpose: strdup() with the added feature of escaping binary
- *              data in the string to safely fit into an SQL query
+ *      Purpose: Esacpe "'" and any other wierd charactors
  *
  *************************************************************************/
 int sql_escape_string(char *to, char *from, int length) {
 
-/*	if(from==NULL) 
-        	return NULL;
-	if((to=malloc((strlen(from)*2) + 1)) == NULL)   
-        	return NULL;
-*/
 	mysql_escape_string(to, from, length);
 	return 1;
 }
